/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.proc.TaskPool.java
 * Created on: May 10, 2017
 */
package fr.bigeon.gclc.proc;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class TaskPool {
    private final Map<String, Task> running = new HashMap<>();
    private int count = 0;
    private final Object lock = new Object();

    public void add(final Task cmd) {
        final String pid = getPID();
        synchronized (lock) {
            running.put(pid, cmd);
        }
        cmd.addInterruptionListener(new InterruptionListener() {

            @Override
            public void interrupted() {
                synchronized (lock) {
                    running.remove(pid);
                    count = Math.min(count, Integer.parseInt(pid));
                }
                cmd.rmInterruptionListener(this);
            }
        });
    }

    /** @return the process id */
    private String getPID() {
        synchronized (lock) {
            String pid;
            do {
                pid = Integer.toString(count++);
            } while (running.containsKey(pid));
            return pid;
        }
    }

    public Task get(String pid) {
        synchronized (lock) {
            return running.get(pid);
        }
    }

    /** @return the pids */
    public Collection<String> getPIDs() {
        return new HashSet<>(running.keySet());
    }
}
