/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.ScriptExecutionTest.java
 * Created on: Jun 12, 2016
 */
package fr.bigeon.gclc.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.fail;

import java.io.IOException;
import java.nio.charset.Charset;

import org.junit.Test;

import fr.bigeon.gclc.ConsoleApplication;
import fr.bigeon.gclc.ConsoleTestApplication;
import fr.bigeon.gclc.exception.CommandRunException;
import fr.bigeon.gclc.exception.CommandRunExceptionType;
import fr.bigeon.gclc.manager.PipedConsoleManager;

/** <p>
 * Test class for {@link ScriptExecution}
 *
 * @author Emmanuel Bigeon */
@SuppressWarnings("static-method")
public class ScriptExecutionTest {

    /**
     * Test method for {@link fr.bigeon.gclc.command.ScriptExecution#execute(java.lang.String[])}.
     */
    @Test
    public void testExecute() {
        PipedConsoleManager test;
        try {
            test = new PipedConsoleManager();
        } catch (IOException e2) {
            fail("creation of console manager failed"); //$NON-NLS-1$
            assertNotNull(e2);
            return;
        }
        ConsoleApplication app = new ConsoleApplication(
                test, "", "");
        new ConsoleTestApplication().attach(app);
        ScriptExecution exec = new ScriptExecution("script", app, "#", //$NON-NLS-1$ //$NON-NLS-2$
                Charset.forName("UTF-8"));
        try {
            exec.execute();
            fail("execution of script command with no file should fail"); //$NON-NLS-1$
        } catch (CommandRunException e1) {
            // ok
            assertEquals(exec, e1.getSource());
            assertEquals(CommandRunExceptionType.USAGE, e1.getType());
        }

        try {
            exec.execute("src/test/resources/scripts/withprependSpace.txt"); //$NON-NLS-1$
            fail("execution of script with lines begining with space should fail"); //$NON-NLS-1$
        } catch (CommandRunException e1) {
            // ok
            assertEquals(exec, e1.getSource());
            assertEquals(CommandRunExceptionType.EXECUTION, e1.getType());
        }

        try {
            exec.execute("src/test/resources/scripts/invalidCmdParse.txt"); //$NON-NLS-1$
            fail("execution of script with invalid command line should fail"); //$NON-NLS-1$
        } catch (CommandRunException e1) {
            // ok
            assertEquals(exec, e1.getSource());
            assertEquals(CommandRunExceptionType.EXECUTION, e1.getType());
        }

        try {
            exec.execute("src/test/resources/scripts/invalidCmd.txt"); //$NON-NLS-1$
            fail("execution of script with invalid command should fail"); //$NON-NLS-1$
        } catch (CommandRunException e1) {
            // ok
            assertEquals(exec, e1.getSource());
            assertEquals(CommandRunExceptionType.EXECUTION, e1.getType());
        }

        try {
            exec.execute("src/test/resources/scripts/failingCmdInvoc.txt"); //$NON-NLS-1$
            fail("execution of script with failing command should fail"); //$NON-NLS-1$
        } catch (CommandRunException e1) {
            // ok
            assertEquals(exec, e1.getSource());
            assertEquals(CommandRunExceptionType.EXECUTION, e1.getType());
        }

        try {
            exec.execute("src/test/resources/scripts/someNonExisting.file"); //$NON-NLS-1$
            fail("execution of script with unexisting file should fail"); //$NON-NLS-1$
        } catch (CommandRunException e1) {
            // ok
            assertEquals(exec, e1.getSource());
            assertEquals(CommandRunExceptionType.EXECUTION, e1.getType());
        }

        try {
            exec.execute("src/test/resources/script1.txt"); //$NON-NLS-1$
            exec.execute("src/test/resources/script2.txt"); //$NON-NLS-1$
            exec.execute("src/test/resources/script3.txt"); //$NON-NLS-1$
            exec.execute("src/test/resources/script4.txt"); //$NON-NLS-1$
            exec.execute("src/test/resources/script5.txt", "test"); //$NON-NLS-1$ //$NON-NLS-2$
        } catch (CommandRunException e) {
            e.printStackTrace();
            fail("execution of wellformed script should not fail"); //$NON-NLS-1$
        }
        try {
            test.close();
        } catch (IOException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /** Test method for {@link fr.bigeon.gclc.command.ScriptExecution#tip()}. */
    @Test
    public void testTip() {
        ScriptExecution exec = new ScriptExecution("script", null, "#", //$NON-NLS-1$ //$NON-NLS-2$
                Charset.forName("UTF-8"));
        exec.tip();
    }

    /** Test method for
     * {@link fr.bigeon.gclc.command.ScriptExecution#help(fr.bigeon.gclc.manager.ConsoleManager, String...)}. */
    @Test
    public void testHelp() {
        ScriptExecution exec = new ScriptExecution("script", null, "#", //$NON-NLS-1$ //$NON-NLS-2$
                Charset.forName("UTF-8"));
        try (PipedConsoleManager test = new PipedConsoleManager()) {
            exec.help(test);
            exec.help(test, "ignored element");
        } catch (IOException e) {
            e.printStackTrace();
            fail("unexpected error in help invocation"); //$NON-NLS-1$
        }
    }
}
